from builtins import str
from builtins import object
import uuid
from . import errors


class Job(object):
    """Class to represent the job part of project being submitted to Envoy service"""

    CREDITS_BUDGET_ACTION_ALERT = 'alert'
    CREDITS_BUDGET_ACTION_STOP = 'terminate'
    CREDITS_BUDGET_ACTION_SUSPEND = 'suspend'

    def __init__(self, name, app, app_version, operation, path, instances, machine_type, *plugins, **params):
        self.id = str(uuid.uuid4())
        self.name = name
        self.app = app
        self.app_version = app_version
        self.operation = operation
        self.path = path
        self.instances = instances if instances > 0 else 0
        self.machine_type = machine_type if machine_type else ''
        self.plugins = plugins if plugins else list()
        self.params = params if params else dict()
        self.dependencies = list()
        self.project = None
        self.machine_type = machine_type
        self.credits_budget_value = 0.0
        self.credits_budget_action = None

    def add_dependencies(self, *jobs):
        self.dependencies.extend(jobs)

    def validate(self):
        # TODO: implement functionality
        return True

    def set_credits_budget(self, value, action):
        val = 0.0
        try:
            val = float(value)
        except:
            raise errors.InvalidArgumentError("Invalid credits budget value")

        if val == 0.0:
            raise errors.InvalidArgumentError("Invalid credits budget value")

        if not action in (Job.CREDITS_BUDGET_ACTION_ALERT, Job.CREDITS_BUDGET_ACTION_STOP, Job.CREDITS_BUDGET_ACTION_SUSPEND):
            raise errors.InvalidArgmentError("Invalid credits budget action")

        self.credits_budget_value = val
        self.credits_budget_action = action

    @property
    def serialize(self):
        job = dict()
        job['id'] = self.id
        job['name'] = self.name
        job['app'] = self.app
        job['app_version'] = self.app_version
        job['operation'] = self.operation
        job['path'] = self.path

        job['plugins'] = dict()
        billing_meta_plugins = list()

        for p in self.plugins:
            items = p.split(':')

            if len(items) == 2:
                job['plugins'][items[0]] = items[1]

            billing_meta_plugins.append(items[0])

        if self.instances:
            job['instances'] = self.instances

        if self.machine_type:
            job['machine_type'] = self.machine_type

        if self.credits_budget_value and self.credits_budget_action:
            job['credits_budget'] = dict()
            job['credits_budget']['value'] = self.credits_budget_value
            job['credits_budget']['action'] = self.credits_budget_action

        job['params'] = self.params
        job['params']['output_dir_name'] = self.project.remote_output_folder_name

        if 'output_upload' not in job['params']: 
            upload_pattern = '{0}/.+'.format(self.project.remote_output_folder)
            job['params']['output_upload'] = list()
            job['params']['output_upload'].append(upload_pattern)
            job['params']['output_upload'].append(upload_pattern.replace('/', '\\\\'))

        job['dependencies'] = [j.id for j in self.dependencies]

        if 'billing_meta' not in job['params']:
            job['params']['billing_meta'] = list()
            job['params']['billing_meta'].append(self.app)
            job['params']['billing_meta'].extend(billing_meta_plugins)

        return job
