from __future__ import absolute_import
from builtins import str
from builtins import object
import datetime
import json
import os
from .watch_file import WatchFile

class Project(object):
    """Class to represent the project being submitted to Envoy service"""

    def __init__(self, project_folder, name=u'', submission_name=u'', output_folder_name=u'', description=u'', **params):
        self.timestamp = datetime.datetime.now()
        self.local_root = os.path.abspath(u'{}'.format(project_folder))
        self.name = u'{}'.format(name) if name else os.path.split(self.local_root)[-1]
        self.submission_name = u'{}'.format(submission_name) if submission_name else self.timestamp.strftime(u"%Y-%b-%d %H_%M_%S")
        self.remote_root = u"/{0}".format(self.name)
        self.params = params if params else dict()
        self.remote_output_folder_name = u'{}'.format(output_folder_name if output_folder_name else self.submission_name.replace(':', '_'))
        self.remote_output_folder = u"{0}/{1}".format(self.remote_root, self.remote_output_folder_name) if output_folder_name else self.remote_root
        self.description = description

        self.jobs = list()
        self.files = list()
        self.watch_files = list()
        self.skip_upload = False
        self.skip_auto_download = False

        # define default watch files
        download_path = os.path.join(self.local_root, u'gm_results', self.submission_name)
        download_pattern = u"{0}/.+".format(self.remote_output_folder)
        self.watch_files_default = WatchFile(download_pattern, download_path)

    def  _is_in_directory(self, path, parent_dir):
        return os.path.commonprefix([path, parent_dir]) == parent_dir

    def _rel_path(self, path, parent_dir):
        return os.path.relpath(path, os.path.dirname(parent_dir))

    def _get_files(self, path):
        files = list()
        if os.path.isdir(path) :
            for dir_name, _, dir_files in os.walk(u'{}'.format(path)):
                for dir_file in dir_files:
                    src_file = os.path.join(dir_name, dir_file)
                    src_file = os.path.relpath(src_file, self.local_root)
                    src_file = src_file.replace("\\", "/")
                    src_file_encoded = src_file.encode('utf-8')
                    files.append(src_file if type(src_file_encoded) == bytes else src_file_encoded)

        return files

    def add_jobs(self, *jobs):
        for j in jobs:
            j.project = self

        self.jobs.extend(jobs)

    def add_files(self, *args):
        for f in args:
            if f not in self.files:
                self.files.append(f)

    def add_watch_files(self, *watch_files):
        self.watch_files.extend(watch_files)

    def validate(self):
        return True

    @property
    def LocalRoot(self):
        return self.local_root

    @property
    def RemoteRoot(self):
        return self.remote_root

    @property
    def serialize(self):
        data = dict()
        data['project_request'] = dict()
        data['watch_files'] = dict()

        data['project_request']['name'] = self.name
        data['project_request']['submission_name'] = self.submission_name
        data['project_request']['description'] = self.description

        data['project_request']['params'] = self.params
        data['project_request']['jobs'] = list()

        for j in self.jobs:
            data['project_request']['jobs'].append(j.serialize)

        data['project_files'] = list()
        data['project_files'].append({
            'localRoot': '',
            'remoteRoot': self.remote_root,
            'files': self.files
        })

        if self.skip_upload:
            data['skip_upload'] = self.skip_upload

        if not self.skip_auto_download:
            watch_files = list()
            watch_files.extend(self.watch_files)

            if not watch_files:
                watch_files.append(self.watch_files_default)

            for item in watch_files:
                data['watch_files'].update(item.serialize)

        # add the watch files also to project request
        data['project_request']['watch_files'] = data.get('watch_files', dict())

        return data

    @property
    def upload_serialize(self):
        data = dict()
        data['upload'] = list()
        data['upload'].append({
            'localRoot': '',
            'remoteRoot': self.remote_root,
            'files': self.files
        })

        return data
